package com.keao.edu.im.mec.im;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.keao.edu.common.exception.BizException;
import com.keao.edu.im.api.entity.BaseMessage;
import com.keao.edu.im.dao.RoomMemberDao;
import com.keao.edu.im.http.HttpHelper;
import com.keao.edu.im.pojo.IMApiResultInfo;
import com.keao.edu.im.pojo.IMTokenInfo;
import com.keao.edu.im.pojo.RoomMember;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.net.HttpURLConnection;
import java.net.URLEncoder;
import java.util.List;

/**
 * Created by weiqinxiao on 2019/2/28.
 */
@Slf4j
@Component
public class IMHelper {
    private static final String UTF8 = "UTF-8";

    @Autowired
    HttpHelper httpHelper;
    @Autowired
    RoomMemberDao roomMemberDao;

    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    /**
     * 获取 Token 方法
     *
     * @param userId:用户 Id，最大长度 64 字节.是用户在 App 中的唯一标识码，必须保证在同一个 App 内不重复，重复的用户 Id 将被当作是同一用户。（必传）
     * @param name:用户名称，最大长度 128 字节.用来在 Push 推送时显示用户的名称.用户名称，最大长度 128 字节.用来在 Push 推送时显示用户的名称。（必传）
     * @param portraitUri:用户头像 URI，最大长度 1024 字节.用来在 Push 推送时显示用户的头像。（必传）
     * @return TokenResult
     **/
    public IMTokenInfo getToken(String userId, String name, String portraitUri) throws Exception {
        if (userId == null) {
            throw new IllegalArgumentException("Paramer 'userId' is required");
        }

        if (name == null) {
            throw new IllegalArgumentException("Paramer 'name' is required");
        }

        if (portraitUri == null) {
            throw new IllegalArgumentException("Paramer 'portraitUri' is required");
        }

        StringBuilder sb = new StringBuilder();
        sb.append("&userId=").append(URLEncoder.encode(userId.toString(), UTF8));
        sb.append("&name=").append(URLEncoder.encode(name.toString(), UTF8));
        sb.append("&portraitUri=").append(URLEncoder.encode(portraitUri.toString(), UTF8));
        String body = sb.toString();
        if (body.indexOf("&") == 0) {
            body = body.substring(1, body.length());
        }

        HttpURLConnection conn = httpHelper.createIMPostHttpConnection("/user/getToken.json", "application/x-www-form-urlencoded");
        httpHelper.setBodyParameter(body, conn);

        return JSON.parseObject(httpHelper.returnResult(conn, body), IMTokenInfo.class);
    }

    /**
     * 创建群组方法（创建群组，并将用户加入该群组，用户将可以收到该群的消息，同一用户最多可加入 500 个群，每个群最大至 3000 人，App
     * 内的群组数量没有限制.注：其实本方法是加入群组方法 /group/join 的别名。）
     *
     * @param userId:要加入群的用户 Id。（必传）
     * @param groupId:创建群组 Id。（必传）
     * @param groupName:群组 Id 对应的名称。（必传）
     * @return CodeSuccessResult
     **/
    public IMApiResultInfo createGroup(String[] userId, String groupId, String groupName)
            throws Exception {
        if (userId == null) {
            throw new IllegalArgumentException("Paramer 'userId' is required");
        }

        if (groupId == null) {
            throw new IllegalArgumentException("Paramer 'groupId' is required");
        }

        if (groupName == null) {
            throw new IllegalArgumentException("Paramer 'groupName' is required");
        }

        StringBuilder sb = new StringBuilder();

        for (int i = 0; i < userId.length; i++) {
            String child = userId[i];
            sb.append("&userId=").append(URLEncoder.encode(child, UTF8));
        }

        sb.append("&groupId=").append(URLEncoder.encode(groupId, UTF8));
        sb.append("&groupName=").append(URLEncoder.encode(groupName, UTF8));
        String body = sb.toString();
        if (body.indexOf("&") == 0) {
            body = body.substring(1, body.length());
        }

        HttpURLConnection conn = httpHelper
                .createIMPostHttpConnection("/group/create.json", "application/x-www-form-urlencoded");
        httpHelper.setBodyParameter(body, conn);

        return JSON.parseObject(httpHelper.returnResult(conn, body), IMApiResultInfo.class);
    }


    /**
     * 将用户加入指定群组，用户将可以收到该群的消息，同一用户最多可加入 500 个群，每个群最大至 3000 人。
     *
     * @param userId:要加入群的用户 Id，可提交多个，最多不超过 1000 个。（必传）
     * @param groupId:要加入的群 Id。（必传）
     * @param groupName:要加入的群 Id 对应的名称。（必传）
     * @return CodeSuccessResult
     **/
    public IMApiResultInfo joinGroup(String[] userId, String groupId, String groupName)
            throws Exception {
        if (userId == null) {
            throw new IllegalArgumentException("Paramer 'userId' is required");
        }

        if (groupId == null) {
            throw new IllegalArgumentException("Paramer 'groupId' is required");
        }

        if (groupName == null) {
            throw new IllegalArgumentException("Paramer 'groupName' is required");
        }

        StringBuilder sb = new StringBuilder();

        for (int i = 0; i < userId.length; i++) {
            String child = userId[i];
            sb.append("&userId=").append(URLEncoder.encode(child, UTF8));
        }

        sb.append("&groupId=").append(URLEncoder.encode(groupId, UTF8));
        sb.append("&groupName=").append(URLEncoder.encode(groupName, UTF8));
        String body = sb.toString();
        if (body.indexOf("&") == 0) {
            body = body.substring(1, body.length());
        }

        HttpURLConnection conn = httpHelper
                .createIMPostHttpConnection("/group/join.json", "application/x-www-form-urlencoded");
        httpHelper.setBodyParameter(body, conn);

        return JSON.parseObject(httpHelper.returnResult(conn, body), IMApiResultInfo.class);
    }



    /**
     * 退出群组方法（将用户从群中移除，不再接收该群组的消息.）
     *
     * @param userId:要退出群的用户 Id.（必传）
     * @param groupId:要退出的群 Id.（必传）
     * @return CodeSuccessResult
     **/
    public IMApiResultInfo quit(String[] userId, String groupId) throws Exception {
        if (userId == null) {
            throw new IllegalArgumentException("Paramer 'userId' is required");
        }

        if (groupId == null) {
            throw new IllegalArgumentException("Paramer 'groupId' is required");
        }

        StringBuilder sb = new StringBuilder();

        for (int i = 0; i < userId.length; i++) {
            String child = userId[i];
            sb.append("&userId=").append(URLEncoder.encode(child, UTF8));
        }

        sb.append("&groupId=").append(URLEncoder.encode(groupId, UTF8));
        String body = sb.toString();
        if (body.indexOf("&") == 0) {
            body = body.substring(1, body.length());
        }

        HttpURLConnection conn = httpHelper
                .createIMPostHttpConnection("/group/quit.json", "application/x-www-form-urlencoded");
        httpHelper.setBodyParameter(body, conn);

        return JSON.parseObject(httpHelper.returnResult(conn, body), IMApiResultInfo.class);
    }


    /**
     * 解散群组方法。（将该群解散，所有用户都无法再接收该群的消息。）
     *
     * @param userId:操作解散群的用户 Id。（必传）
     * @param groupId:要解散的群 Id。（必传）
     * @return CodeSuccessResult
     **/
    public IMApiResultInfo dismiss(String userId, String groupId) throws Exception {
        if (userId == null) {
            throw new IllegalArgumentException("Paramer 'userId' is required");
        }

        if (groupId == null) {
            throw new IllegalArgumentException("Paramer 'groupId' is required");
        }

        StringBuilder sb = new StringBuilder();
        sb.append("&userId=").append(URLEncoder.encode(userId, UTF8));
        sb.append("&groupId=").append(URLEncoder.encode(groupId, UTF8));
        String body = sb.toString();
        if (body.indexOf("&") == 0) {
            body = body.substring(1, body.length());
        }

        HttpURLConnection conn = httpHelper
                .createIMPostHttpConnection("/group/dismiss.json", "application/x-www-form-urlencoded");
        httpHelper.setBodyParameter(body, conn);

        return JSON.parseObject(httpHelper.returnResult(conn, body), IMApiResultInfo.class);
    }

    /**
     * 开始录制
     * @param roomId
     * @param registrationId
     * @return
     * @throws Exception
     */
    public void startRecord(String roomId, Long registrationId) throws Exception {
        JSONObject paramJson = new JSONObject();
        paramJson.put("sessionId",roomQuery(roomId));

        JSONObject config = new JSONObject();
        config.put("mode",3);
        config.put("videoFormat","mp4");
        config.put("audioFormat","aac");
        config.put("videoResolution","640x480");
        config.put("mixLayout",1);
        config.put("sliceMin",30);
        config.put("renderMode",1);
        config.put("hostUserId","");
        config.put("hostStreamId","");
        againRecord(paramJson,registrationId,roomId,config);
    }

    @Async
    public void againRecord(JSONObject paramJson,Long registrationId,String roomId,JSONObject config) throws Exception {
        List<RoomMember> roomMembers = roomMemberDao.findByRidAndMsidFlag(roomId,true);
        if (roomMembers == null || roomMembers.size() == 0) {
            log.error("Paramer 'roomMembers' is can not be null");
            return;
        }
        config.put("input",getInput(roomMembers));
        paramJson.put("config",config);
        String body = paramJson.toJSONString();
        HttpURLConnection conn = httpHelper.createIMRtcPostHttpConnection("/rtc/record/start.json", "application/json",roomId);
        httpHelper.setBodyParameter(body, conn);
        IMApiResultInfo resultInfo = JSON.parseObject(httpHelper.returnResult(conn, body), IMApiResultInfo.class);
        if(resultInfo.getResultCode() == 10000){
            redisTemplate.opsForValue().set(resultInfo.getRecordId(),registrationId.toString());
        }else if(resultInfo.getResultCode() == 47032 || resultInfo.getResultCode() == 47002){
            log.error("直播视频录制失败：body : {},resultInfo : {} 准备重试",body,resultInfo);
            int i = 1;
            while (i<=3){
                redisTemplate.delete("sessionId:" + roomId);
                Thread.sleep(2000l);
                roomMembers = roomMemberDao.findByRidAndMsidFlag(roomId,true);
                if (roomMembers == null || roomMembers.size() == 0) {
                    log.error("Paramer 'roomMembers' is can not be null");
                    i++;
                    continue;
                }
                if(i == 3){
                    config.put("input",getSignInput(roomMembers));
                }else {
                    config.put("input",getInput(roomMembers));
                }
                paramJson.put("config",config);
                body = paramJson.toJSONString();
                log.error("直播视频录制第{}次重试：body : {}",i,body);
                HttpURLConnection connection = httpHelper.createIMRtcPostHttpConnection("/rtc/record/start.json", "application/json", roomId);
                httpHelper.setBodyParameter(body, connection);
                IMApiResultInfo imApiResultInfo = JSON.parseObject(httpHelper.returnResult(connection, body), IMApiResultInfo.class);
                if(imApiResultInfo.getResultCode() == 10000){
                    redisTemplate.opsForValue().set(imApiResultInfo.getRecordId(),registrationId.toString());
                    log.info("直播视频录制第{}次重试成功",i);
                    return;
                }
                if(i == 3){
                    log.error("直播视频录制失败：resultInfo : {} 重试结束",imApiResultInfo);
                    return;
                }
                i++;
            }
        }
    }

    /**
     * 更改录制布局
     * @param roomId
     * @return
     * @throws Exception
     */
    @Async
    public IMApiResultInfo configRecord(String roomId) throws Exception {
        if (roomId == null) {
            throw new IllegalArgumentException("Paramer 'roomId' is required");
        }
        List<RoomMember> members = roomMemberDao.findByRidAndMsidFlag(roomId,true);
        if (members == null || members.size() == 0) {
            throw new IllegalArgumentException("Paramer 'roomMembers' is can not be null");
        }
        JSONObject paramJson = new JSONObject();
        paramJson.put("sessionId",roomQuery(roomId));

        paramJson.put("mixLayout",1);
        paramJson.put("renderMode",1);
        paramJson.put("hostUserId","");
        paramJson.put("hostStreamId","");

        paramJson.put("input",getInput(members));
        String body = paramJson.toJSONString();

        HttpURLConnection conn = httpHelper.createIMRtcPostHttpConnection("/rtc/record/config.json", "application/json",roomId);
        httpHelper.setBodyParameter(body, conn);
        IMApiResultInfo resultInfo = JSON.parseObject(httpHelper.returnResult(conn, body), IMApiResultInfo.class);
        return resultInfo;
    }

    private JSONObject getInput(List<RoomMember> roomMembers){

        JSONArray videos = new JSONArray();
        boolean isMaster = true;

        int size = roomMembers.size();
        float x = -0.2f;
        int num = 1;
        for (int i = 0; i < size; i++) {
            RoomMember roomMember = roomMembers.get(i);
            JSONObject video = new JSONObject();
            video.put("userId",roomMember.getUid());
            video.put("streamId",roomMember.getUid() + "_RongCloudRTC");
            if((roomMember.getRole() == 3 && isMaster) || (i == size - 1 && isMaster) || (i == 5 && isMaster)){
                video.put("x",0);
                video.put("y",0);
                if(size == 1){
                    video.put("width",1);
                    video.put("height",1);
                }else {
                    video.put("width",1);
                    video.put("height",0.8);
                }
                isMaster = false;
            }else {
                video.put("width",0.2);
                video.put("height",0.2);
                if(num <= 5){
                    x = x + 0.2f;
                    video.put("x",x);
                    video.put("y",0.8);
                }else {
                    break;
                }
                num++;
            }
            videos.add(video);
        }
        JSONObject input = new JSONObject();
        input.put("video",videos);
        return input;
    }

    private JSONObject getSignInput(List<RoomMember> roomMembers){

        JSONArray videos = new JSONArray();
        int size = roomMembers.size();
        for (int i = 0; i < size; i++) {
            RoomMember roomMember = roomMembers.get(i);
            JSONObject video = new JSONObject();
            video.put("userId",roomMember.getUid());
            video.put("streamId",roomMember.getUid() + "_RongCloudRTC");
            if(roomMember.getRole() == 3 || i == size - 1){
                video.put("x",0);
                video.put("y",0);
                video.put("width",1);
                video.put("height",1);
                videos.add(video);
                break;
            }
        }
        JSONObject input = new JSONObject();
        input.put("video",videos);
        return input;
    }

    /**
     * 结束录制
     * @param roomId
     * @return
     * @throws Exception
     */
    @Async
    public IMApiResultInfo stopRecord(String roomId) throws Exception {
        if (roomId == null) {
            throw new IllegalArgumentException("Paramer 'roomId' is required");
        }
        JSONObject paramJson = new JSONObject();
        paramJson.put("sessionId",roomQuery(roomId));

        String body = paramJson.toJSONString();

        HttpURLConnection conn = httpHelper.createIMRtcPostHttpConnection("/rtc/record/stop.json", "application/json",roomId);
        httpHelper.setBodyParameter(body, conn);
        redisTemplate.delete("sessionId:" + roomId);
        return JSON.parseObject(httpHelper.returnResult(conn, body), IMApiResultInfo.class);
    }

    /**
     * 查询房间状态
     * @param roomId
     * @return
     * @throws Exception
     */
    public String roomQuery(String roomId){
        if (StringUtils.isEmpty(roomId)) {
            throw new IllegalArgumentException("Paramer 'roomId' is required");
        }
        String sessionId = redisTemplate.opsForValue().get("sessionId:" + roomId);
        if(StringUtils.isNotEmpty(sessionId)){
            return sessionId;
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("roomId",roomId);

        HttpURLConnection conn = null;
        try {
            conn = httpHelper.createIMRtcPostHttpConnection("/rtc/room/query.json", "application/json",null);
            httpHelper.setBodyParameter(jsonObject.toJSONString(), conn);
            String returnResult = httpHelper.returnResult(conn, jsonObject.toJSONString());
            JSONObject resultObject = JSONObject.parseObject(returnResult);
            String code = resultObject.get("code").toString();
            if("200".equals(code)){
                sessionId = resultObject.get("sessionId").toString();
                redisTemplate.opsForValue().set("sessionId:" + roomId,sessionId);
            }else {
                log.error("获取sessionId失败 returnResult:{}",returnResult);
                throw new BizException("获取sessionId失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return sessionId;
    }


    /**
     * 发送群组消息方法（以一个用户身份向群组发送消息，单条消息最大 128k.每秒钟最多发送 20 条消息，每次最多向 3 个群组发送，如：一次向 3 个群组发送消息，示为 3 条消息。）
     *
     * @param fromUserId:发送人用户 Id 。（必传）
     * @param toGroupId:接收群Id，提供多个本参数可以实现向多群发送消息，最多不超过 3 个群组。（必传）
     * @param pushContent:定义显示的 Push 内容，如果 objectName 为融云内置消息类型时，则发送后用户一定会收到 Push 信息. 如果为自定义消息，则
     * pushContent 为自定义消息显示的 Push 内容，如果不传则用户不会收到 Push 通知。（可选）
     * @param pushData:针对 iOS 平台为 Push 通知时附加到 payload 中，Android 客户端收到推送消息时对应字段名为 pushData。（可选）
     * @param isPersisted:当前版本有新的自定义消息，而老版本没有该自定义消息时，老版本客户端收到消息后是否进行存储，0 表示为不存储、 1 表示为存储，默认为 1
     * 存储消息。（可选）
     * @param isCounted:当前版本有新的自定义消息，而老版本没有该自定义消息时，老版本客户端收到消息后是否进行未读消息计数，0 表示为不计数、 1 表示为计数，默认为 1
     * 计数，未读消息数增加 1。（可选）
     * @return CodeSuccessResult
     **/
    public IMApiResultInfo publishMessage(String fromUserId, String toGroupId, BaseMessage message) throws Exception {
        String[] toGroupIds = new String[1];
        toGroupIds[0] = toGroupId;
        return publishMessage(fromUserId, null, toGroupIds, message, "", "", 0,
                0, 0, 0, 0);
    }

    public IMApiResultInfo publishMessage(String fromUserId, String toGroupId, BaseMessage message, Integer isIncludeSender) throws Exception {
        String[] toGroupIds = new String[1];
        toGroupIds[0] = toGroupId;
        return publishMessage(fromUserId, null, toGroupIds, message, "", "", 0,
                0, isIncludeSender, 0, 0);
    }

    //定向消息 toUserId 指向对应的人
    public IMApiResultInfo publishMessage(String fromUserId, String toUserId, String toGroupId, BaseMessage message) throws Exception {
        String[] toGroupIds = new String[1];
        toGroupIds[0] = toGroupId;
        return publishMessage(fromUserId, toUserId, toGroupIds, message, "", "", 0,
                0, 0, 0, 0);
    }

    public IMApiResultInfo publishMessage(String fromUserId, String toUserId, String[] toGroupId,
                                          BaseMessage message, String pushContent, String pushData, Integer isPersisted,
                                          Integer isCounted, Integer isIncludeSender, Integer isStatus, Integer isMentioned)
            throws Exception {
        if (fromUserId == null) {
            throw new IllegalArgumentException("Paramer 'fromUserId' is required");
        }

        if (toGroupId == null) {
            throw new IllegalArgumentException("Paramer 'toGroupId' is required");
        }

        if (message == null) {
            throw new IllegalArgumentException("Paramer 'message' is required");
        }

        StringBuilder sb = new StringBuilder();
        sb.append("&fromUserId=").append(URLEncoder.encode(fromUserId, UTF8));

        //定向消息
        if (toUserId != null) {
            sb.append("&toUserId=").append(URLEncoder.encode(toUserId, UTF8));
        }

        for (int i = 0; i < toGroupId.length; i++) {
            String child = toGroupId[i];
            sb.append("&toGroupId=").append(URLEncoder.encode(child, UTF8));
        }

        String msgStr = message.toString();
        log.info("publish msg: {}", msgStr);
        sb.append("&objectName=").append(URLEncoder.encode(message.getObjectName(), UTF8));
        sb.append("&content=").append(URLEncoder.encode(msgStr, UTF8));

        if (pushContent != null) {
            sb.append("&pushContent=").append(URLEncoder.encode(pushContent, UTF8));
        }

        if (pushData != null) {
            sb.append("&pushData=").append(URLEncoder.encode(pushData, UTF8));
        }

        if (isPersisted != null) {
            sb.append("&isPersisted=").append(URLEncoder.encode(isPersisted.toString(), UTF8));
        }

        if (isCounted != null) {
            sb.append("&isCounted=").append(URLEncoder.encode(isCounted.toString(), UTF8));
        }

        if (isIncludeSender != null) {
            sb.append("&isIncludeSender=")
                    .append(URLEncoder.encode(isIncludeSender.toString(), UTF8));
        }

        if (isMentioned != null) {
            sb.append("&isMentioned=").append(URLEncoder.encode(isMentioned.toString(), UTF8));
        }

        String body = sb.toString();
        if (body.indexOf("&") == 0) {
            body = body.substring(1, body.length());
        }

        String url;
        if (isStatus != null && isStatus.intValue() == 1) {
            url = "/statusmessage/group/publish.json";
        } else {
            url = "/message/group/publish.json";
        }

        HttpURLConnection conn = httpHelper
                .createIMPostHttpConnection(url, "application/x-www-form-urlencoded");
        httpHelper.setBodyParameter(body, conn);

        return JSON.parseObject(httpHelper.returnResult(conn, body), IMApiResultInfo.class);
    }
}
