package com.keao.edu.user.controller;

import com.alibaba.fastjson.JSONArray;
import com.keao.edu.common.controller.BaseController;
import com.keao.edu.common.entity.HttpResponseResult;
import com.keao.edu.common.exception.BizException;
import com.keao.edu.common.page.PageInfo;
import com.keao.edu.thirdparty.adapay.ConfigInit;
import com.keao.edu.thirdparty.adapay.CorpMember;
import com.keao.edu.thirdparty.adapay.Payment;
import com.keao.edu.user.dao.SysConfigDao;
import com.keao.edu.user.entity.HfMember;
import com.keao.edu.user.page.HfMemberQueryInfo;
import com.keao.edu.user.service.HfMemberService;
import com.keao.edu.util.date.DateUtil;
import com.keao.edu.util.excel.POIUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.IOUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@RequestMapping("adapay")
@Api(tags = "汇付服务")
@RestController
public class AdapayController extends BaseController {

    @Autowired
    private HfMemberService hfMemberService;

    @ApiOperation(value = "企业用户列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('adapay/queryPage')")
    public HttpResponseResult<PageInfo<HfMember>> queryPage(HfMemberQueryInfo queryInfo) {
        return succeed(hfMemberService.queryPage(queryInfo));
    }

    @ApiOperation("新建企业用户")
    @PostMapping(value = "createMember")
    @PreAuthorize("@pcs.hasPermissions('adapay/createMember')")
    public HttpResponseResult<Map<String, Object>> createMember(HfMember member) throws Exception {
        if (member.getMultipartFile().isEmpty()) {
            return failed("证件压缩文件必传");
        }
        File file = new File("/var/tmp/" + member.getMultipartFile().getOriginalFilename());
        InputStream inputStream = member.getMultipartFile().getInputStream();

        try {
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
            FileOutputStream fos = new FileOutputStream(file);
            IOUtils.copy(inputStream, fos);
            return succeed(hfMemberService.createMember(member, file));
        } catch (Exception e) {
            return failed(e.getMessage());
        } finally {
            IOUtils.closeQuietly(inputStream);
            boolean delete = file.delete();
        }
    }

    @ApiOperation("修改企业用户(状态是failed才能修改)")
    @PostMapping(value = "updateMember")
    @PreAuthorize("@pcs.hasPermissions('adapay/updateMember')")
    public HttpResponseResult<Map<String, Object>> updateMember(HfMember member) throws Exception {
        if (member.getMultipartFile().isEmpty()) {
            return failed("证件压缩文件必传");
        }
        File file = new File("/var/tmp/" + member.getMultipartFile().getOriginalFilename());
        InputStream inputStream = member.getMultipartFile().getInputStream();

        try {
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
            FileOutputStream fos = new FileOutputStream(file);
            IOUtils.copy(inputStream, fos);
            return succeed(hfMemberService.updateMember(member, file));
        } catch (Exception e) {
            return failed(e.getMessage());
        } finally {
            IOUtils.closeQuietly(inputStream);
            boolean delete = file.delete();
        }
    }


    @ApiOperation("绑定结算账户（succeeded 状态才能使用）")
    @PostMapping(value = "createSettleAccount")
    @PreAuthorize("@pcs.hasPermissions('adapay/createSettleAccount')")
    public HttpResponseResult<Map<String, Object>> createSettleAccount(String memberId, String cardNo, String bankCode) throws Exception {
        try {
            return succeed(hfMemberService.createSettleAccount(memberId, cardNo, bankCode));
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @ApiOperation("查询企业用户信息")
    @GetMapping(value = "getMemberInfo")
    @PreAuthorize("@pcs.hasPermissions('adapay/getMemberInfo')")
    public HttpResponseResult<Map<String, Object>> getMemberInfo(String memberId) throws Exception {
        return succeed(CorpMember.executeQueryMember(memberId));
    }


    @ApiOperation("导出对账单")
    @GetMapping(value = "exportBill")
    @PreAuthorize("@pcs.hasPermissions('adapay/exportBill')")
    public void exportBill(Date startTime, Date endTime, HttpServletResponse response) throws Exception {

        long createdGte = startTime.getTime();
        long createdLte = DateUtil.getLastSecondWithDay(endTime).getTime();
        int pageIndex = 1;
        List<Map<String, Object>> data = new ArrayList<>();
        while (true) {
            Map<String, Object> paymentList = Payment.queryList(pageIndex, createdGte, createdLte);
            JSONArray payments = (JSONArray) paymentList.get("payments");
            if (!paymentList.get("status").equals("succeeded")) {
                throw new BizException("查询失败，请重试");
            }

            if (payments != null && payments.size() > 0) {
                for (Object payment : payments) {
                    Map<String, Object> paymentMap = (Map<String, Object>) payment;
                    paymentMap.put("created_time", DateUtil.timeStamp2Date(paymentMap.get("created_time").toString(), null));
                    if (paymentMap.get("pay_channel").equals("alipay_qr")) {
                        paymentMap.put("pay_channel", "支付宝正扫");
                    } else if (paymentMap.get("pay_channel").equals("alipay_wap")) {
                        paymentMap.put("pay_channel", "支付宝H5支付");
                    } else {
                        paymentMap.put("pay_channel", "微信公众号支付");
                    }

                    if (paymentMap.get("status").equals("pending")) {
                        paymentMap.put("status", "交易处理中");
                    } else if (paymentMap.get("status").equals("succeeded")) {
                        paymentMap.put("status", "交易成功");
                    } else {
                        paymentMap.put("status", "交易失败");
                    }
                    JSONArray divMembers = (JSONArray) paymentMap.get("div_members");
                    for (Object divMember : divMembers) {
                        Map<String, Object> divMemberMap = (Map<String, Object>) divMember;
                        divMemberMap.putAll(paymentMap);
                        if (divMemberMap.get("member_id").equals("0")) {
                            divMemberMap.put("member_id", ConfigInit.merNo);
                        }
                        if (divMemberMap.get("fee_flag").equals("N")) {
                            divMemberMap.put("fee_amt", "0.00");
                        }
                        data.add(divMemberMap);
                    }
                }
            }
            if (paymentList.get("has_more").equals(false)) {
                break;
            }
            pageIndex++;
        }

        OutputStream outputStream = response.getOutputStream();
        try {
            String[] header = {"支付流水号", "订单号", "支付渠道", "交易金额", "商户号", "分润金额", "是否承担手续费", "手续费", "第三方订单号", "支付宝/微信订单号", "交易时间", "交易状态"};
            String[] body = {"id", "order_no", "pay_channel", "pay_amt", "member_id", "amount", "fee_flag", "fee_amt", "party_order_id", "out_trans_id", "created_time", "status"};

            HSSFWorkbook workbook = POIUtil.exportExcel(header, body, data);
            response.setContentType("application/octet-stream");
            response.setHeader("Content-Disposition", "attachment;filename=bill-" + DateUtil.getDate(new Date()) + ".xls");
            response.flushBuffer();
            outputStream = response.getOutputStream();
            workbook.write(outputStream);
            outputStream.flush();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
